"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Nano = require("nano");
const util_1 = require("util");
const BulkTool_1 = require("./BulkTool");
const AggregateView_1 = require("./View/AggregateView");
const ViewEngine_1 = require("./View/ViewEngine");
const nano = Nano;
const COUCH_USER = process.env.COUCH_USER || 'admin';
const COUCH_PASSWORD = process.env.COUCH_PASSWORD || 'password';
const COUCH_PORT = process.env.COUCH_PORT || '5984';
const COUCH_HOST = process.env.COUCH_HOST || 'couchdb';
const COUCH_PROTO = process.env.COUCH_PROTO || 'http';
const CONNECTION_STRING = `${COUCH_PROTO}://${COUCH_USER}:${COUCH_PASSWORD}@${COUCH_HOST}:${COUCH_PORT}`;
class Instance {
    static get Instance() {
        if (!this._instance) {
            this._instance = nano.default ? nano.default(CONNECTION_STRING) : nano(CONNECTION_STRING);
        }
        return this._instance;
    }
}
class CouchFactory {
    create(databaseName) {
        return __awaiter(this, void 0, void 0, function* () {
            const connection = Instance.Instance;
            const dbName = databaseName;
            try {
                const dbList = yield connection.db.list();
                if (Array.isArray(dbList)) {
                    if (!DatabaseExists(dbName, dbList)) {
                        yield connection.db.create(dbName);
                    }
                    const db = connection.use(dbName);
                    const newProvider = new CouchProvider(db);
                    return newProvider;
                }
                else {
                    throw new Error(`An unexpected error occurred while trying to list the available CouchDB databases.`);
                }
            }
            catch (e) {
                console.error(util_1.inspect(e, false, null, true));
                throw new Error(`Failed to connect the ${databaseName} CouchDB Database.
      ${e}`);
            }
        });
    }
}
exports.CouchFactory = CouchFactory;
class CouchProvider {
    constructor(db) {
        this.db = db;
        this.bulkTool = new BulkTool_1.BulkToolProvider(this.db);
    }
    get(id) {
        return this.db.get(id);
    }
    insert(data, id) {
        if (id) {
            return this.db.insert(data, id);
        }
        return this.db.insert(data);
    }
    update(data, id, rev) {
        data._id = id;
        data._rev = rev;
        return this.db.insert(data);
    }
    bulkUpdate(docs) {
        return this.bulkTool.bulkUpdate(docs);
    }
    delete(id, rev) {
        return this.db.destroy(id, rev);
    }
    find(query) {
        return this.db.find(query);
    }
    view(designDocument, view, options) {
        return options ? this.db.view(designDocument, view, options) : this.db.view(designDocument, view);
    }
    getBulkTool() {
        return this.bulkTool;
    }
    viewEngine(designDocument, aggregateViews) {
        const views = aggregateViews.map(view => AggregateView_1.AggregateViewFactory.Create(view));
        const engine = new ViewEngine_1.AggregateViewEngine(this.db, designDocument, views);
        return engine;
    }
}
exports.CouchProvider = CouchProvider;
function DatabaseExists(databaseName, availableDatabases) {
    return availableDatabases.includes(databaseName);
}
