import { DocumentViewParams } from 'nano';
import { BulkUpdateResponseList, IBulkToolProvider } from '../BulkTool';
import { IAggregateView } from '../View/AggregateView';
import { AggregateViewEngine } from '../View/ViewEngine';
export interface ICouchDocumentMetaData {
    _id: string;
    _rev: string;
}
export interface ICouchDocumentData {
    [key: string]: any;
}
export interface ITypedDocument {
    type: string;
}
export interface ICouchDocument extends ICouchDocumentMetaData, ICouchDocumentData {
}
export interface ICouchTypedDocument extends ICouchDocument, ITypedDocument {
}
export declare type CouchDocument = ICouchDocument | ICouchTypedDocument;
export declare const BAD_REQUEST = "bad_request";
export declare const NOT_FOUND = "not_found";
export declare const CONFLICT = "conflict";
export declare const INVALID_REV_FORMAT = "Invalid rev format";
export declare const DOCUMENT_UPDATE_CONFLICT = "Document update conflict.";
export interface IDocumentModificationResult {
    ok: boolean;
    id: string;
    rev: string;
}
export interface IFindResults {
    docs: CouchDocument[];
    bookmark: string;
    warning?: string;
}
export interface IReducedViewResult {
    key: any;
    value: any;
}
export interface IViewIndexItem {
    key: any;
    value: any;
}
export interface IViewResult {
    id: string;
    key: any;
    value: any;
}
export interface IFetchResponse {
    total_rows: number;
    offset: number;
    rows: ICouchDocument[];
}
export interface IViewResultWithDoc extends IViewResult {
    doc: CouchDocument;
}
export declare type ViewResultType = IViewResult | IViewResultWithDoc;
export interface IViewResultResponse {
    total_rows: number;
    offset: number;
    rows: ViewResultType[];
}
export interface IReducedViewResultResponse {
    rows: IReducedViewResult[];
}
export declare type ViewResultResponse = IViewResultResponse | IReducedViewResultResponse;
export interface ICouchProvider {
    get(id: string): Promise<CouchDocument>;
    insert(data: any, id?: string): Promise<IDocumentModificationResult>;
    update(data: any, id: string, rev: string): Promise<IDocumentModificationResult>;
    bulkUpdate(docs: ICouchDocument[]): Promise<BulkUpdateResponseList>;
    delete(id: string, rev: string): Promise<IDocumentModificationResult>;
    find(query: IMangoQuery): Promise<IFindResults>;
    getBulkTool(): IBulkToolProvider;
    viewEngine(designDocument: string, aggregateViews: IAggregateView[]): AggregateViewEngine;
    view(designDocument: string, view: string, options?: DocumentViewParams): Promise<ViewResultResponse>;
}
export interface ICouchFactory {
    create(databaseName: string): Promise<ICouchProvider>;
}
export interface IDocumentViewParams {
    descending?: boolean;
    endkey?: any;
    group?: boolean;
    group_level?: number;
    include_docs?: boolean;
    inclusive_end?: boolean;
    key?: any;
    keys?: any[];
    limit?: number;
    reduce?: boolean;
    skip?: number;
    startkey?: any;
}
export declare type MangoValue = number | string | Date | boolean | {
    [key: string]: any;
};
export declare enum SortDirection {
    ASC = "asc",
    DESC = "desc"
}
export declare type SortItem = string | {
    [key: string]: SortDirection;
};
export declare type SortOptions = SortItem[];
export interface ISortOptions {
    sort?: SortOptions;
}
export declare type MangoOperatorValue = IMangoSelector | MangoValue | MangoValue[];
export interface IMangoSelector {
    [key: string]: MangoOperatorValue;
}
export interface IMangoQuery {
    selector: IMangoSelector;
    limit?: number;
    skip?: number;
    sort?: SortOptions;
    fields?: string[];
    use_index?: string | [string, string];
}
