"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const TypeChecks_1 = require("../Utils/TypeChecks");
function SumValues(values) {
    const valueCount = values.length;
    if (TypeChecks_1.IsObject(values[0])) {
        throw new Error('Reducing object values is currently not supported');
    }
    else {
        const sum = [0];
        for (let i = 0; i < valueCount; i++) {
            const value = values[i];
            if (TypeChecks_1.IsArray(value)) {
                const subValueCount = value.length;
                for (let j = 0; j < subValueCount; j++) {
                    const subValue = value[j];
                    if (!TypeChecks_1.IsNumber(subValue)) {
                        throw new Error('The built-in sum reducer requires number values');
                    }
                    if (!sum[j])
                        sum[j] = 0;
                    sum[j] += subValue;
                }
            }
            else {
                if (!TypeChecks_1.IsNumber(value)) {
                    throw new Error('The built-in sum reducer requires number values');
                }
                sum[0] += value;
            }
        }
        return sum.length === 1 ? sum[0] : sum;
    }
}
exports.SumValues = SumValues;
function Count(keys, values, rereduce) {
    if (rereduce) {
        return SumValues(values);
    }
    else {
        return values.length;
    }
}
exports.Count = Count;
function Stats(keys, values, rereduce) {
    if (rereduce) {
        return {
            sum: values.reduce((a, b) => {
                return a + b.sum;
            }, 0),
            min: values.reduce((a, b) => {
                return Math.min(a, b.min);
            }, Infinity),
            max: values.reduce((a, b) => {
                return Math.max(a, b.max);
            }, -Infinity),
            count: values.reduce((a, b) => {
                return a + b.count;
            }, 0),
            sumsqr: values.reduce((a, b) => {
                return a + b.sumsqr;
            }, 0),
        };
    }
    else {
        return {
            sum: SumValues(values),
            min: Math.min.apply(null, values),
            max: Math.max.apply(null, values),
            count: values.length,
            sumsqr: (() => {
                let sumsqr = 0;
                values.forEach(value => {
                    sumsqr += value * value;
                });
                return sumsqr;
            })(),
        };
    }
}
exports.Stats = Stats;
function Sum(keys, values) {
    return SumValues(values);
}
exports.Sum = Sum;
