"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Interfaces_1 = require("../Design/Interfaces");
const MockView_1 = require("./MockView");
class MockDesign {
    constructor(name, views) {
        this.name = name;
        this.views = views;
    }
    insert(doc) {
        for (const view of this.views.values()) {
            view.map(doc);
        }
    }
    update(doc) {
        for (const view of this.views.values()) {
            view.update(doc);
        }
    }
    delete(id) {
        for (const view of this.views.values()) {
            view.delete(id);
        }
    }
    view(viewName, options) {
        const view = this.views.get(viewName);
        if (!view) {
            throw new Error(`No view named: ${viewName} exists in the ${this.name} design document`);
        }
        return view.query(options);
    }
    static Generate(manifest) {
        const designMap = new Map();
        for (const designDocName of Object.keys(manifest)) {
            const designDoc = manifest[designDocName];
            if (designDoc.type === Interfaces_1.DesignDocType.JAVASCRIPT) {
                const existingDesignDoc = designMap.get(designDocName);
                if (existingDesignDoc) {
                    throw new Error(`Duplicate design doc detected! Its name is: ${designDocName}`);
                }
                const viewMap = BuildJavaScriptDesignDoc(designDoc);
                designMap.set(designDocName, new MockDesign(designDocName, viewMap));
            }
        }
        return designMap;
    }
}
exports.MockDesign = MockDesign;
function BuildJavaScriptDesignDoc(manifest) {
    const viewMap = new Map();
    for (const viewName of Object.keys(manifest.views)) {
        const view = manifest.views[viewName];
        const mockView = new MockView_1.MockView(view);
        viewMap.set(viewName, mockView);
    }
    return viewMap;
}
