"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
class ViewEngine {
}
exports.ViewEngine = ViewEngine;
class AggregateViewEngine extends ViewEngine {
    constructor(db, designDocument, aggregateViews) {
        super();
        this.db = db;
        this.designDocument = designDocument;
        this.aggregateMap = new Map(aggregateViews.map((_a) => {
            var { name } = _a, props = __rest(_a, ["name"]);
            return [name, Object.assign({ name }, props)];
        }));
    }
    getAggregateById(name, id) {
        return __awaiter(this, void 0, void 0, function* () {
            const config = this.aggregateMap.get(name);
            if (!config) {
                throw new Error(`The view engine does not contain a configuration for an aggregate named: ${name}.`);
            }
            const { childMappings } = config;
            const maxJoinIndex = childMappings.length;
            try {
                const document = yield this.db.view(this.designDocument, name, {
                    start_key: [id],
                    end_key: [id, maxJoinIndex],
                    include_docs: true,
                });
                return this.parseAggregateResponse(config, document);
            }
            catch (e) {
                console.log(e);
                throw new Error(`There was an error while trying to get an Aggregate with an ID of ${id} from the ${name} View`);
            }
        });
    }
    getAggregatesInList(name, ids) {
        return __awaiter(this, void 0, void 0, function* () {
            const config = this.aggregateMap.get(name);
            if (!config) {
                throw new Error(`The view engine does not contain a configuration for an aggregate named: ${name}.`);
            }
            const { childMappings } = config;
            const maxJoinIndex = childMappings.length;
            const keys = [];
            for (const id of ids) {
                const keySet = [];
                for (let i = 0; i < maxJoinIndex + 1; i++) {
                    if (i === 0) {
                        keySet[i] = [id];
                    }
                    else {
                        keySet[i] = [id, i];
                    }
                }
                keys.push(keySet);
            }
            const flatKeys = [].concat(...keys);
            try {
                const documents = yield this.db.view(this.designDocument, name, {
                    keys: flatKeys,
                    include_docs: true,
                });
                return this.parseAggregateResponseList(config, documents);
            }
            catch (e) {
                console.log(e);
                throw new Error(`There was an error while trying to get a list of Aggregates with the following IDs: ${ids.toString} from the ${name} View`);
            }
        });
    }
    parseAggregateResponse(config, response) {
        const { childMappings, rootForeignKey } = config;
        const { rows } = response;
        const _a = rows.shift().doc, { _id, _rev, type } = _a, root = __rest(_a, ["_id", "_rev", "type"]);
        const result = Object.assign({}, root);
        const keyMap = {};
        for (let _b of childMappings) {
            const { key, type: mappingType } = _b, options = __rest(_b, ["key", "type"]);
            result[key] = [];
            keyMap[mappingType] =
                options && options.rootForeignKey ? { key, rootForeignKey: options.rootForeignKey } : { key };
        }
        for (let _c of rows) {
            const _d = _c.doc, { _id: childId, _rev: childRev, type: childType } = _d, _e = rootForeignKey, fk = _d[_e], child = __rest(_d, ["_id", "_rev", "type", typeof _e === "symbol" ? _e : _e + ""]);
            const { key, rootForeignKey: removedKey } = keyMap[childType];
            if (removedKey) {
                const _f = removedKey, removedForeignKey = child[_f], theRest = __rest(child, [typeof _f === "symbol" ? _f : _f + ""]);
                result[key].push(Object.assign({}, theRest));
            }
            else {
                result[key].push(Object.assign({}, child));
            }
        }
        return result;
    }
    parseAggregateResponseList(config, responses) {
        const { childMappings, rootForeignKey } = config;
        const { rows } = responses;
        const keyMap = {};
        for (let _a of childMappings) {
            const { key, type: mappingType } = _a, options = __rest(_a, ["key", "type"]);
            keyMap[mappingType] =
                options && options.rootForeignKey ? { key, rootForeignKey: options.rootForeignKey } : { key };
        }
        const results = [];
        let currentResult = null;
        for (let _b of rows) {
            const _c = _b.doc, { _id, _rev, type } = _c, document = __rest(_c, ["_id", "_rev", "type"]);
            if (keyMap[type]) {
                if (keyMap[type].rootForeignKey && document[keyMap[type].rootForeignKey]) {
                    const { [keyMap[type].rootForeignKey]: removedKey } = keyMap[type];
                    const _d = removedKey, removedForeignKey = document[_d], theRest = __rest(document, [typeof _d === "symbol" ? _d : _d + ""]);
                    currentResult[keyMap[type].key].push(theRest);
                }
                else if (document[rootForeignKey]) {
                    const _e = rootForeignKey, removedForeignKey = document[_e], theRest = __rest(document, [typeof _e === "symbol" ? _e : _e + ""]);
                    currentResult[keyMap[type].key].push(theRest);
                }
                else {
                    if (currentResult) {
                        results.push(currentResult);
                    }
                    currentResult = Object.assign({}, document);
                    for (const child of childMappings) {
                        currentResult[child.key] = [];
                    }
                }
            }
            else {
                if (currentResult) {
                    results.push(currentResult);
                }
                currentResult = Object.assign({}, document);
                for (const child of childMappings) {
                    currentResult[child.key] = [];
                }
            }
        }
        results.push(currentResult);
        return results;
    }
}
exports.AggregateViewEngine = AggregateViewEngine;
